from dataclasses import dataclass

from ..result import Result, Success, Failure
from ..transformer import Transformer


@dataclass
class EndTransform(Transformer[None]):
    def __init__(self) -> None:
        pass
    
    def parse(self, stream: str, at_index: int) -> Result[None]:
        if len(stream) == at_index:
            return Success(None, at_index)
        return Failure(at_index, set(["<end>"]))


from okipy.lib import test


@test()
def test_end_simple(ctx):
    parser = EndTransform()
    input = ""
    assert parser.parse(input, 0) == Success(None, 0)

@test()
def test_end_shifted(ctx):
    parser = EndTransform()
    input = ".."
    assert parser.parse(input, 2) == Success(None, 2)

@test()
def test_end_fails(ctx):
    parser = EndTransform()
    input = "arbre......"
    assert parser.parse(input, 2) == Failure(2, set(["<end>"]))
