from dataclasses import dataclass

from ..result import Result, Success, Failure
from ..transformer import Transformer


@dataclass
class JustTransform(Transformer[str]):
    word: str

    def parse(self, stream: str, at_index: int) -> Result[str]:
        end_index = at_index + len(self.word)
        prefix = stream[at_index:end_index]
        if prefix == self.word:
            return Success(self.word, end_index)
        return Failure(at_index, set([self.word]))


from okipy.lib import test


@test()
def test_just_simple(ctx):
    parser = JustTransform("arbre")
    input = "arbre"
    assert parser.parse(input, 0) == Success("arbre", 5)

@test()
def test_just_shifted(ctx):
    parser = JustTransform("arbre")
    input = "..arbre..."
    assert parser.parse(input, 2) == Success("arbre", 7)

@test()
def test_just_fails(ctx):
    parser = JustTransform("arbre")
    input = "arbre......"
    assert parser.parse(input, 2) == Failure(2, set(["arbre"]))
