from dataclasses import dataclass
from typing import TypeVar, Union

from ..result import Result, Success, Failure
from ..transformer import Transformer


L = TypeVar("L")
R = TypeVar("R")
@dataclass
class OrTransform(Transformer[Union[L, R]]):
    left: Transformer[L]
    right: Transformer[R]

    def parse(self, stream: str, at_index: int) -> Result[Union[L, R]]:
        result_left = self.left.parse(stream, at_index)
        if isinstance(result_left, Success):
            return Success(result_left.value, result_left.next_index)
        result_right = self.right.parse(stream, at_index)
        if isinstance(result_right, Success):
            return Success(result_right.value, result_right.next_index)
        return Failure(at_index, set(), result_left, result_right)


from okipy.lib import test
from .just import JustTransform


@test()
def test_or_simple(ctx):
    parser = OrTransform(JustTransform("ar"), JustTransform("bre"))
    input = "arbre"
    assert parser.parse(input, 0) == Success("ar", 2)

@test()
def test_or_other(ctx):
    parser = OrTransform(JustTransform("ar"), JustTransform("bre"))
    input = "brear"
    assert parser.parse(input, 0) == Success("bre", 3)

@test()
def test_or_shifted(ctx):
    parser = OrTransform(JustTransform("ar"), JustTransform("bre"))
    input = "..arbre..."
    assert parser.parse(input, 2) == Success("ar", 4)

@test()
def test_or_fail(ctx):
    parser = OrTransform(JustTransform("ar"), JustTransform("bre"))
    input = "......"
    assert parser.parse(input, 2) == Failure(2, set(["ar", "bre"]), Failure(2, set(["ar"])), Failure(2, set(["bre"])))
